#include "CamImage.h"
#include "FWlibrary/labImage.h"
#include <stdio.h>
#include <iostream>
#include "ImprovMain.h"

#include "movies.h"
#include "Camera.h"

extern char *path;
extern int subscale;
extern int factor;
extern char **files;
extern char *copy;
extern int files_num;

static Improv_Menu_Item menu_items[] = 
{  
  //Standard Scenes 
  {"Camera On Off",E_CameraOn_Off, 0},
    
  {"Sequences/Taxi",E_Taxi, 0},
  {"Sequences/Trees",E_Tree, 0},
  {"Sequences/Pipes",E_Pipes, 0},
  //{"Standard/Coke",E_Coke, 0},
  //{"Standard/Taxi",E_Taxi, 0},
  
  //{"Synthetic/Trans Cube",E_Trans_Cube, 0},
  //{"Synthetic/Office",E_Office, 0},
  //{"Synthetic/Pipes",E_Pipes, 0},
  //{"Synthetic/Box",E_Box, 0},
};


// IMPROV configuration structure
IPL_CONFIG ipl_config;
CamImage::CamImage(int x,int y,int w,int h,const char *label)
            : Fl_Group(x,y,w,h)
{
  //Title
  menu = new Fl_Menu_Button(x+65, y, 105, 20, "Original");
  menu->box(FL_THIN_UP_BOX);
  menu->labelsize(12);
  menu->textsize(12);
  init_menu();

  //Display Area
  box = new Fl_Box(x, y+20,w,h-45, label);
  
  //DisplayImage Offset
  ipl_config.display_xOffset=3;
  ipl_config.display_yOffset=23;

  //Final display Size
  ipl_config.displayWidth=w-2*ipl_config.display_xOffset;
  ipl_config.displayHeight=h-5-2*ipl_config.display_yOffset;
  
  box->box(FL_ENGRAVED_BOX);
  box->labelsize(12);
  box->align(FL_ALIGN_TOP_LEFT);
  
  initialised = false;
  stopped = true;
  
  enlarge=true;
 
  camera = NULL;
}

static int fake_camera=-1;
static int fake_cam=0;
int which_cam=0;
extern int nr_cameras;
void CamImage::initialize(Picture *p_out, bool *colour)
{
  Camera **camp;
  
  if (NULL != camera)
    camera->close();
  
  camera = NULL;
  //Which Camera to try first?
  if(which_cam==1)
    {
      camp = cameras;
      camp+=nr_cameras-3;
      for (;NULL != camp; camp++)
	{
	  if ( (*camp)->open(&ipl_config) )
	    {
	      camera = *camp;
	      break;
	    }
	  //fprintf(stdout,"Failed, trying next device...\n");
	  fprintf(stdout,"Failed!\n"); exit(1);
	}
    } //FakeCam
  else if(which_cam==2)
    {
      camp = cameras;
      camp+=nr_cameras-2;
      
      for (;NULL != camp; camp++)
	{
	  if ( (*camp)->open(&ipl_config) )
	    {
	      camera = *camp;
	      fake_camera=1;
	      fake_cam=1;
	      break;
	    } 
	  //fprintf(stdout,"Failed, trying next device...\n"); 
	  fprintf(stdout,"Failed!\n"); exit(1);
	}
    }//V4L2
  else if(which_cam==3)
    {
      camp = cameras;
      camp+=nr_cameras-4;
      
      for (;NULL != camp; camp++)
	{
	  if ( (*camp)->open(&ipl_config) )
	    {
	      camera = *camp;
	      break;
	    } 
	  //fprintf(stdout,"Failed, trying next device...\n"); 
	  fprintf(stdout,"Failed!\n"); exit(1);
	}
    }
  else
    {
      for (camp = cameras; NULL != camp; camp++)
	{
	  if ( (*camp)->open(&ipl_config) )
	    {
	      camera = *camp;
	      break;
	    }
	}
    }
   
  if (NULL == camera)
    {
      cout << "No camera found, not even the fake one.  Exiting!" << endl;
      exit(1);
    }
  
  cam_colour = camera->iscolor();
  
  *colour = cam_colour;
  
  //Initiate camera output image
  setImageType(p_out,ipl_config.imageType);
  resizeImage(p_out,ipl_config.imageWidth,ipl_config.imageHeight);
  
  //Set global variables for improc.c
  ipl_config.colorBlack=0;
  ipl_config.colorWhite=255;
  IPL_init(&ipl_config);
  
  p_output_img=p_out;
  
  initialised = true;
}


CamImage::~CamImage()
{
  if (NULL != camera)
    camera->close();
  
  delete menu;
    delete box;
}

void CamImage::camera_stopped(bool state)
{
  stopped = state;
}

void CamImage::enable_enlarge()
{
  enlarge = false;
  subscale = 0;
}

void CamImage::set_params(double p[])
{
  camera->params(p);  
}

void CamImage::disable_enlarge()
{
    enlarge = true;
    subscale = 1;
}


void CamImage::save_image(char *fname)
{
  saveImage(fname,p_output_img);
}

void CamImage::get_cam_info(CamInfo *info)
{
  camera->get_info(info);
}

void CamImage::get_IPL_info()
{
  Camera **camp;
  
  if (NULL != camera)
    camera->close();
  
  camera = NULL;
  
  for (camp = cameras; NULL != camp; camp++)
    {
      if ((*camp)->get_IPL(&ipl_config))
	{
	  (*camp)->close();
	  break;
	}
    }
}

#ifndef MAX
#define MAX(a,b) ((a)>=(b)?(a):(b))
#endif
#ifndef MIN
#define MIN(a,b) ((a)>=(b)?(b):(a))
#endif
int restart=0; 
int cameraon_off=1;
void CamImage::draw()
{

  if (initialised) 
    camera->read(p_output_img);
  
  Fl_Group::draw();
  
  if(cameraon_off==1)
    {
      if (p_output_img->format==pix_rgb24) 
	{
	  if(( p_output_img->width  > ipl_config.displayWidth ||
	       p_output_img->height > ipl_config.displayHeight ) && 
	     enlarge==true )
	    {
	      Picture *tmp;
	      
	      tmp=cropImage(p_output_img,0,0, 
			    MIN(ipl_config.displayWidth,p_output_img->width), 
			    MIN(ipl_config.displayHeight,p_output_img->height));
	      
	      fl_draw_image(tmp->data,
			    x()+ipl_config.display_xOffset,
			    y()+ipl_config.display_yOffset,
			    tmp->width,
			    tmp->height, 3, 0);
	      
	      freeImage(tmp);
	      
	    }
	    else
	      fl_draw_image(p_output_img->data,
			    x()+ipl_config.display_xOffset,
			    y()+ipl_config.display_yOffset,
			    p_output_img->width,
			    p_output_img->height,3,0);
	}
      else 
	{ 
	  fl_draw_image_mono(p_output_img->data,
			     x()+ipl_config.display_xOffset,
			     y()+ipl_config.display_yOffset,
			     p_output_img->width,
			     p_output_img->height,1,0);
	}
    }
}


//Camera menu
#ifndef _ARRAY_SIZE
#define ARRAY_SIZE(a) (int)(sizeof(a)/sizeof((a)[0]))
#endif
void CamImage::menu_cb(Fl_Widget *w,void *param)
{
  Improv_Menu_Item *item=(Improv_Menu_Item *) param;;
  IPLOP op;

  op.operation = item->operation;
  
  switch(op.operation)
    {
    case(E_CameraOn_Off):
      {	
	if(fake_camera==1 && fake_cam==1)
	  {
	    Camera *tmp;
	    tmp=cameras[0];  
	    cameras[0]=cameras[nr_cameras-2];
	    cameras[nr_cameras-2]=tmp;
	    fake_cam=0;
	    fake_camera=0;
	    cameraon_off=1;
	    restart=1;
	  }
	else
	  cameraon_off = !cameraon_off;
      }break;
      
    case(E_Tree):
      {
	//Last Camera should always be the 'fakecamera'
	//Now switching last with first!
	if(fake_cam==0)
	  { 
	    Camera *tmp;
	    
	    cameras[0]->close();
	    tmp=cameras[0];  
	    cameras[0]=cameras[nr_cameras-2];
	    cameras[nr_cameras-2]=tmp;
	    
	    fake_camera=1; 
	    fake_cam=1;
	  }
	
	factor=2;
	path="tree/";
	copy="SRI Tree Sequence:\n Courtesy of SRI International via Sarnoff Research Center";

	printf(copy);

	files=tree;
	files_num=ARRAY_SIZE(tree);
	//Indicate that Improv has to be re-initialised
	restart=1;
      } break;
      
    case(E_Coke):
      {
	//Last Camera should always be the 'fakecamera'
	if(fake_cam==0)
	  { 
	    Camera *tmp;
	    
	    tmp=cameras[0];  
	    cameras[0]=cameras[nr_cameras-2];
	    cameras[nr_cameras-2]=tmp;
	    
	    fake_camera=1;
	    fake_cam=1;
	  }
	
	factor=2;
	path="NASA_DATA/";
	copy="NASA COKE Sequence:\nCourtesy of SRI International\nvia Sarnoff Research Center"; printf(copy);
	files=coke;
	files_num=ARRAY_SIZE(coke);

	restart=1;
      } break;
    
    case(E_Taxi):
      {
	//Last Camera should always be the 'fakecamera'
	if(fake_cam==0)
	  { 
	    Camera *tmp;
	    tmp=cameras[0];  
	    cameras[0]=cameras[nr_cameras-2];
	    cameras[nr_cameras-2]=tmp;
	    
	    fake_camera=1;
	    fake_cam=1;
	  }
	
	factor=2;
	path="TAXI_DATA/";
	copy="Hamburg Taxi Sequence:\nCourtesy of the University of Hamburg, Germany";
	files=taxi;printf(copy);
	files_num=ARRAY_SIZE(taxi);

	restart=1;
      } break; 
 
    case(E_Office):
      {//Last Camera should always be the 'fakecamera'
	if(fake_cam==0)
	  { 
	    Camera *tmp;
	    tmp=cameras[0];  
	    cameras[0]=cameras[nr_cameras-2];
	    cameras[nr_cameras-2]=tmp;
	    fake_camera=1;
	    fake_cam=1;
	  }
	//subscale=0;
	factor=2;
	path="office/";
	
	files=office;
	files_num=ARRAY_SIZE(office);
	copy="Office Sequence:\nCourtesy of the Department of Computer Science,\nUniversity of Otago, New Zealand";printf(copy);
	restart=1;
      } break;
      
    case(E_Trans_Cube):
      {//Last Camera should always be the 'fakecamera'
	if(fake_cam==0)
	  { 
	    Camera *tmp;
	    tmp=cameras[0];  
	    cameras[0]=cameras[nr_cameras-2];
	    cameras[nr_cameras-2]=tmp;
	    fake_camera=1;
	    fake_cam=1;
	  }
	//subscale=0;
	factor=3;
	path="cube/";
	copy="SOFA synthetic sequences:\nCourtesy of the Computer Vision Group, Heriot-Watt University\n(http://www.cee.hw.ac.uk/~mtc/sofa)";
	files=trans_cube; printf(copy);
	files_num=ARRAY_SIZE(trans_cube);
	
	restart=1;
      } break;
      
    case(E_Pipes):
      {//Last Camera should always be the 'fakecamera'
	if(fake_cam==0)
	  { 
	    Camera *tmp;
	    tmp=cameras[0];  
	    cameras[0]=cameras[nr_cameras-2];
	    cameras[nr_cameras-2]=tmp;
	    fake_camera=1;
	    fake_cam=1;
	  }
	//subscale=0;
	factor=2;
	path="S5/";
	copy="SOFA synthetic sequences:\nCourtesy of the Computer Vision Group, Heriot-Watt University\n(http://www.cee.hw.ac.uk/~mtc/sofa)";printf(copy);
	
	files=pipes;
	files_num=ARRAY_SIZE(pipes);
	
	restart=1;
      } break;
      
    case(E_Box):
      {//Last Camera should always be the 'fakecamera'
	if(fake_cam==0)
	  { 
	    Camera *tmp;
	    tmp=cameras[0];  
	    cameras[0]=cameras[nr_cameras-2];
	    cameras[nr_cameras-2]=tmp;
	    fake_camera=1;
	    fake_cam=1;
	  }
	//subscale=0;
	factor=2;
	path="S2/";
	copy="SOFA synthetic sequences:\n Courtesy of the Computer Vision Group, Heriot-Watt University\n(http://www.cee.hw.ac.uk/~mtc/sofa)";printf(copy);
	files=trans_box;
	files_num=ARRAY_SIZE(trans_box);
	
	restart=1;
      } break;
      
    default: break;
      
    }
  
  op.num_param = item->num_param;
  
}

void CamImage::init_menu()
{
    Improv_Menu_Item *item;
    int menu_size = sizeof(menu_items)/sizeof(Improv_Menu_Item);
    int i;

    for (i=0; i<menu_size; i++) 
      {
	item = &menu_items[i];
	menu->add(item->text, 0, (CamImage::menu_cb), item,0);
      }
}







