#include "ImprovMain.h"
#include "Camera.h"
//#include "EyeCam.h"
//#include "EyeCamOld.h"
#include "QuickCam.h" 

#include "FWlibrary/labImage.h"
#include "FWlibrary/labImgDraw.h"

#include <string>

#if defined(HAVE_V4L)
#include "V4LCamera.h"
#endif

#define ARRAY_SIZE(a) (int)(sizeof(a)/sizeof((a)[0]))

#if defined(HAVE_V4L2)
#include "V4L2Camera.h"
#endif

FakeCamera fakecamera;

Camera *cameras[] = {
#if defined(HAVE_V4L)
  //&v4lcamera,
#endif
#if defined(HAVE_V4L2)
  &v4l2camera,
#endif
  &quickcam,
  &fakecamera,
  NULL
};

//Number of cameras incl. NULL !IMPORTANT!
int nr_cameras=ARRAY_SIZE(cameras);

//default image sequence
#define PATH "tree/"
#define SUBSCALE 1 
#define FACTOR 2
static char *fake_files[] = {
  "trees.-1.raw.pgm", "trees.0.raw.pgm", "trees.1.raw.pgm","trees.2.raw.pgm",
  "trees.3.raw.pgm", "trees.4.raw.pgm", "trees.5.raw.pgm", "trees.6.raw.pgm",
  "trees.7.raw.pgm", "trees.8.raw.pgm", "trees.9.raw.pgm", "trees.10.raw.pgm",
  "trees.11.raw.pgm", "trees.12.raw.pgm", "trees.13.raw.pgm",
  "trees.14.raw.pgm", "trees.15.raw.pgm", "trees.16.raw.pgm",
  "trees.17.raw.pgm", "trees.18.raw.pgm", "trees.19.raw.pgm" 
};
#define ARRAY_SIZE(a) (int)(sizeof(a)/sizeof((a)[0]))
#ifndef MAX
#define MAX(a,b) ((a)>=(b)?(a):(b))
#endif
#ifndef MIN
#define MIN(a,b) ((a)>=(b)?(b):(a))
#endif
int subscale=SUBSCALE;
int factor=FACTOR;
char *copy="SRI Tree Sequence:\nCourtesy of SRI International via Sarnoff Research Center";
char *path=PATH;
char **files=fake_files;
int files_num=ARRAY_SIZE(fake_files);

extern char *img_path;
extern int restart;

int fake_num_alt;
bool
FakeCamera::open(IPL_CONFIG *ipl_config)
{
  int tmp=0;  
  extern ImprovUI *improv;
  
  fake_num = ARRAY_SIZE(fake_files);
  fake_num = files_num;
  
  if(!restart)
   {
     test_sequence = new Picture*[fake_num];
     fake_num_alt=fake_num;
   }  
  else
    {
    for(int i=0; i < fake_num_alt; i++)
      { 
	freeImage(test_sequence[i]);
      }
    
    test_sequence = new Picture*[fake_num];
    fake_num_alt=fake_num;
    }
  
  fprintf(stdout,"\n");
  fprintf(stdout,"Loading Imagesequence...\n");
  fprintf(stdout,"------------------------\n");
  fprintf(stdout,copy);
  fprintf(stdout,"\n");
  // Load test sequence
  for(int i=0; i < fake_num; i++)
    { 
      //string s = path;      
      string s = img_path; 
      s+=path;
      s+=files[i];

      test_sequence[i] = newImage();
      //fprintf(stdout,"Loading Image: %s\n",s.c_str());    
      fprintf(stderr,"#");
      if(!loadImage((char *)s.c_str(), test_sequence[i]))
	{
	  fprintf(stderr, "Can not find image!: %s \n",s.c_str());
	  tmp=1;
	}
      
      //Scale images to fit
      if(subscale)
	{
	  scaleImage(test_sequence[i],
		     MIN(ipl_config->displayWidth,
			 test_sequence[i]->width),
		     MIN(ipl_config->displayHeight,
			 test_sequence[i]->height ));
	}
      
      if(test_sequence[i]->width > ipl_config->displayWidth)
	test_sequence[i]=cropImage(test_sequence[i],0,0,
				   ipl_config->displayWidth, 
				   test_sequence[i]->height);
      
      if(test_sequence[i]->height > ipl_config->displayHeight)
	test_sequence[i]=cropImage(test_sequence[i],0,0,
				   test_sequence[i]->width,
				   ipl_config->displayHeight);
      
      
      //Init ipl_config
      ipl_config->imageWidth=test_sequence[0]->width;
      ipl_config->imageHeight=test_sequence[0]->height;
      ipl_config->imageType=test_sequence[0]->format;
      ipl_config->image_ratio = (float) ((float)ipl_config->imageHeight / 
					 (float) ipl_config->imageWidth);
      
      if(test_sequence[i]->width!=ipl_config->imageWidth || 
	 test_sequence[i]->height!=ipl_config->imageHeight )
	fprintf(stderr,"Not same size! \n");
    }             

  //fprintf(stdout,"\n");
  //fprintf(stdout,copy);
  fprintf(stdout,"\n");
  
  // if(tmp==1)
  //{
  //  fprintf(stdout,"One or more image files could not be found!\n" );
  //  fprintf(stdout,"Make sure the enviroment variable $IMPROVSEQ is set\n" );
  //  fprintf(stdout,"or specify a path for the images with 'improv -p[path]'!\n" );
  // }
  
  printf(copy);
  fake_index = 0;
  forward=1;

  ipl_config->colorBlack = DEFAULT_BLACK;
  ipl_config->colorWhite = DEFAULT_WHITE;
    
  return 1;
}

void
FakeCamera::close(void)
{
}

int forward=1;
void
FakeCamera::read(Picture *p_frame)
{
  //Play sequence forth and back
  if(forward)
    fake_index++;
  else
    fake_index--;
  
  if(fake_index>=fake_num-1)
    forward=0;
  else if (fake_index<=0)
    forward=1;
  
  if(p_frame->format==pix_rgb24)
    {
      memcpy((BYTE *)p_frame->data,
	     (BYTE *)test_sequence[fake_index]->data,
	     test_sequence[fake_index]->datasize);
    }
  else if(p_frame->format==pix_grey)
    {    
      memcpy((BYTE *)p_frame->data,
	     (BYTE *)test_sequence[fake_index]->data,
	     test_sequence[fake_index]->datasize);
	     
    }
  
  frame_count++;
}

bool
FakeCamera::iscolor(void)
{
  if(test_sequence[0]->format!=pix_grey)
    return true;
  else
    return false;
}

void
FakeCamera::get_info(CamInfo *info)
{
  if(test_sequence[0]->data!=NULL)
    {
      info->width = test_sequence[0]->width;
      info->height = test_sequence[0]->height;
      
      if(test_sequence[0]->format==pix_grey)
	info->bpp = 1;
      else
	info->bpp = 3;
    }
}

bool
FakeCamera::get_IPL(IPL_CONFIG *ipl_config)
{
  Picture *p_temp;

  string s= path;
  s += fake_files[0];
  
  p_temp=newImage();

  if(!loadImage_Info((char *)s.c_str(), p_temp))
    fprintf(stdout,"Dammed!");
  
  if(subscale)
    subscaleImage(p_temp,factor);
  
  
  if(p_temp->width > ipl_config->displayWidth)
    p_temp=cropImage(p_temp,0,0,
		     ipl_config->displayWidth, 
		     p_temp->height);
  
  if(p_temp->height > ipl_config->displayHeight)
    p_temp=cropImage(p_temp,0,0,
		     p_temp->width,
		     ipl_config->displayHeight); 
  
  ipl_config->imageWidth = p_temp->width;
  ipl_config->imageHeight= p_temp->height;
  ipl_config->imageType = p_temp->format;
  ipl_config->image_ratio = (float) ((float) p_temp->height / 
				     (float) p_temp->width);
  
  freeImage(p_temp);
 
  return 1;

}

void FakeCamera::params(double *p)
{
}







