#include "Camera.h"
#include "EyeCam.h"

EyeCamera eyecamera;

void
EyeCamera::read(Picture *p_frame)
{
  BYTE *frame;
  
  switch (mode) {
    case EYECAM_8_BITS: {
      unsigned raw_data[VV6300_8_NUM_RAW];
      int p1, p2, p3;
      int i, j;
        
      ::read(camera_file, raw_data, VV6300_8_NUM_RAW);

      p1 = 320; p2 = p1 + 160; p3 = VV6300_8_WIDTH;

      // clear top and bottom borders
      memset(frame, VV6300_8_BLACK, 3 * VV6300_8_WIDTH);
      memset(&frame[VV6300_8_WIDTH*(VV6300_8_HEIGHT-1)],
             VV6300_8_BLACK, 3 * VV6300_8_WIDTH);

      for (i=1; i<VV6300_8_HEIGHT-1; i++) {
        // left border pixel
        frame[p3++] = VV6300_8_BLACK;

        // image pixels
        for (j=1; j<VV6300_8_WIDTH-1; j++) {
	  frame[p3++] = raw_data[p1++];    // red pixel
	  frame[p3++] = raw_data[p1++];    // green pixel
	  p2++;
	  frame[p3++] = raw_data[p2++];    // blue pixel
        }

        // right border pixel
        frame[p3++] = VV6300_8_BLACK;
	
	/**********/
	//Something should happen here

	      
      }
      break;
    }

    case EYECAM_4_BITS:
    default:
      /* Currently unsupported */
      break;
  }
}

bool
EyeCamera::open(IPL_CONFIG *ipl_config)
{
  // try 8-bit vv6300 driver
  camera_file = ::open("/dev/vv6300_8", O_RDONLY);
  if (camera_file != -1) {
    mode = EYECAM_8_BITS;

    ipl_config->colorBlack = VV6300_8_BLACK;
    ipl_config->colorWhite = VV6300_8_WHITE;
    ipl_config->imageWidth = VV6300_8_WIDTH;
    ipl_config->imageHeight = VV6300_8_HEIGHT;

    return true;
  }


  // try 4-bit vv6300 driver
  camera_file = ::open("/dev/vv6300_4", O_RDONLY);
  if (camera_file != -1) {
    mode = EYECAM_4_BITS;

    ipl_config->colorBlack = VV6300_4_BLACK;
    ipl_config->colorWhite = VV6300_4_WHITE;
    ipl_config->imageWidth = VV6300_4_WIDTH;
    ipl_config->imageHeight = VV6300_4_HEIGHT;

    return true;
  }

  return false;
}

void
EyeCamera::close(void)
{
  if (-1 != camera_file)
    ::close(camera_file);
  camera_file = -1;
}

bool
EyeCamera::iscolor(void)
{
  return true;
}


void
EyeCamera::get_info(CamInfo *info)
{
  info->width = VV6300_8_WIDTH;
  info->height = VV6300_8_HEIGHT;
  info->bpp = VV6300_8_BPP;
}

bool 
EyeCamera:: get_IPL(IPL_CONFIG *ipl_config)
{
}

void EyeCamera::params(double *p)
{
}
